import { ref } from 'vue'
import type { Options } from '@/utils/mqtt'
import { MqttService } from '@/utils/mqtt'
import { useUserStore } from '@/store/moules/userStore/index'
import { useMessageStore } from '@/store/moules/messageStore/index'
import { MessageStatusEnum, MessageTypeEnum, ModelTypeEnum } from '@/enums/messageEnum'
import type { MqttMessageResult } from '/#/mqtt.d'

export function useMqtt() {
  const userStore = useUserStore()
  const messageStore = useMessageStore()

  const messageData = ref('')

  const options: Options = {
    host: import.meta.env.VITE_GLOB_MQTT_HOST,
    port: import.meta.env.VITE_GLOB_MQTT_PORT,
    protocol: import.meta.env.VITE_GLOB_MQTT_PROTOCOL,
    clean: true,
    connectTimeout: 4000, // 超时时间
    reconnectPeriod: 4000, // 重连时间间隔
    clientId: `Consumer@${userStore.getUserInfo?.chatCode}_${new Date().getTime()}`,
    username: import.meta.env.VITE_GLOB_MQTT_USERNAME,
    password: import.meta.env.VITE_GLOB_MQTT_PASSWORD,
  }

  const mqttService = new MqttService(options)

  /**
   * @description: 订阅
   */
  const subscribe = (topicKey: string) => {
    mqttService
      .subscribe(topicKey)
      .then(() => {
        mqttService.onMessage(topicKey, (message: MqttMessageResult) => {
          if (!messageStore.getConversationData) {
            return
          }
          if (message.conversationId !== messageStore.getConversationData.id) {
            return
          }
          // console.log(message)
          // console.log(topicKey, message.message_content.length)

          if (message.message_type === MessageStatusEnum.ACTICON) {
            // 这里主要是知识库的数据更新，因为知识库回答是每次都是一个字，并不是通常的不断叠加，需要前端手动叠加
            if (
              messageStore.getLastMessageList.messageStatus === MessageStatusEnum.ACTICON
              && Number(messageStore.getConversationData.modelType) === ModelTypeEnum.QANYTHING) {
              messageStore.setMessageStatus(MessageStatusEnum.ACTICON)
              messageStore.setMessageLastPushItem(message.message_content)
            }
            else {
              // 这里是通用的数据更新
              messageStore.setMessageStatus(MessageStatusEnum.ACTICON)
              messageStore.setMessageLastItem({
                messageType: MessageTypeEnum.AI,
                content: message.message_content,
                time: String(new Date().getTime()),
                avatar: '',
                messageStatus: MessageStatusEnum.ACTICON,
              })
            }
          }
          if (message.message_type === MessageStatusEnum.END) {
            messageStore.setMessageStatus(MessageStatusEnum.END)
            messageStore.setMessageLastItem({
              messageType: MessageTypeEnum.AI,
              content: message.message_content,
              time: String(new Date().getTime()),
              avatar: '',
              messageStatus: MessageStatusEnum.END,
            })
            userStore.getChatInfoFun()
          }
        })
      })
  }

  /**
   * @description: 建立连接
   */
  const connect = async () => {
    return new Promise((resolve, reject) => {
      mqttService
        .connect()
        .then(() => {
          const topicKey = `chatgpt_${userStore.getChatInfo?.chatCode}_${messageStore.conversationData?.id}`
          messageStore.setTopicKey(topicKey)
          subscribe(topicKey)
          resolve(true)
        })
        .catch(() => {
          // eslint-disable-next-line prefer-promise-reject-errors
          reject(false)
        })
    })
  }

  /**
   * @description: 取消订阅
   */
  const unsubscribe = (topicKey: string) => {
    mqttService.unsubscribe(topicKey)
  }

  /**
   * @description: 关闭连接
   */
  const end = () => {
    console.log('关闭连接')

    mqttService.end()
  }

  return {
    messageData,
    connect,
    subscribe,
    unsubscribe,
    end,
  }
}
